import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.imageio.*;

/**
 * パネルクラス
 */
public class ARPanel extends JPanel {

	/** キーアダプター */
	private		MGKeyAdapter		mgka		= null;

	/** キーを押している状態一覧 */
	private		boolean[]			keyPressTbl	= null;

	/** 宇宙船位置 */
	private		int					x			= 0;
	private		int					y			= 0;

	/** 速度 */
	private		int					mx			= 0;

	/** 宇宙船 */
	private		BufferedImage[]		imageShips	= null;

	/** 宇宙船の向き */
	private		int					muki		= 0;

	/** タイマー */
	private		java.util.Timer		timerThis	= null;

	/** 経過時間 */
	private		int					time		= 0;

	/**
	 * コンストラクタ
	 */
	public ARPanel() {

		// スーパークラスを呼び出す
		super();

		try{

			// パネルサイズ
			super.setPreferredSize(new Dimension(800, 800));
			// レイアウト設定
			super.setLayout(null);

			// キー押下状況一覧を生成
			keyPressTbl = new boolean[256];

			// 宇宙船を読み込む
			imageShips = new BufferedImage[3];
			InputStream is0 = this.getClass().getResourceAsStream("imageShip0.gif");
			imageShips[0] = ImageIO.read(is0);
			is0.close();

			InputStream is1 = this.getClass().getResourceAsStream("imageShipR.gif");
			imageShips[1] = ImageIO.read(is1);
			is1.close();

			InputStream is2 = this.getClass().getResourceAsStream("imageShipL.gif");
			imageShips[2] = ImageIO.read(is2);
			is1.close();

			// キーアダプタを生成
			mgka = new MGKeyAdapter();
			// パネルにキーリスナーを追加する
			this.addKeyListener(mgka);

			// 初期化処理
			init();

			// タイマーを生成
			timerThis = new java.util.Timer();

			// タイマーをスタート
			timerThis.scheduleAtFixedRate(new TimerActionListener(), 1000l, 8l);

		}catch(Exception ex){
			ex.printStackTrace();
			JOptionPane.showMessageDialog(this, "ERROR : " + ex.toString());
		}

	} // end ARPanel

	/**
	 * 初期化処理
	 */
	public void init(){

		// 時間
		time = 0;

		// 位置
		x = 384;
		y = 640;

	} // end init

	/**
	 * 実行
	 */
	public void run(){

		// 時間＋１
		time++;

		// 宇宙船移動タイミング
		if(time % 6 == 0){

			// 向きを初期化
			muki = 0;

			// 左の場合、
			if(isKeyCodePressed(KeyEvent.VK_LEFT)){
				// 左向きにする
				muki = 2;
				// 加速する
				mx = mx - 1;
			}

			// 右の場合、
			if(isKeyCodePressed(KeyEvent.VK_RIGHT)){
				// 左向きにする
				muki = 1;
				// 加速する
				mx = mx + 1;
			}

		} // end if 宇宙船移動タイミング

		// 移動タイミング
		if(time % 2 == 0){

			// 移動する
			x = x + mx;

			// 一番左に来た場合、
			if(x < 0){
				x = 0;
				mx = 0;

			// 一番右に来た場合、
			}else if(x > this.getWidth() - 32){
				x = this.getWidth() - 32;
				mx = 0;
			}

		// 表示タイミング
		}else{

			// 再描画
			repaint();

		}

	} // end run

	/**
	 * 描画メソッド
	 * <pre>
	 * ペイントする必要があるときにこのメソッドを呼び出されます。
	 * </pre>
	 */
	public void paint(Graphics g){

		// 画面を塗りつぶす
		g.setColor(Color.black);
		g.fillRect(0, 0, 800, 800);

		// 宇宙船を描画する
		g.drawImage(imageShips[muki], x, y, 32, 40, this);

	} // end paint

	/**
	 * キー押下中かをどうかをチェック
	 */
	public boolean isKeyCodePressed(int keyCode){

		return keyPressTbl[keyCode];

	} // end isKeyCodePressed

	/**
	 * キーアダプタークラス
	 */
	private class MGKeyAdapter extends KeyAdapter {

		/**
		 * キーを押したときに呼ばれます
		 */
		@Override
		public void keyPressed(KeyEvent ke){
			int code = ke.getKeyCode();
			if(code < 256){
				keyPressTbl[code] = true;
			}
		}

		/**
		 * キーを離したときに呼ばれます
		 */
		@Override
		public void keyReleased(KeyEvent ke){
			int code = ke.getKeyCode();
			if(code < 256){
				keyPressTbl[code] = false;
			}
		}

	} // MGKeyAdapter

	/**
	 * タイマークラス
	 */
	private class TimerActionListener extends TimerTask {

		/**
		 * 実行
		 */
		@Override
		public void run(){

			// 実行メソッド呼び出し
			ARPanel.this.run();

		} // end actionPerformed

	} // end TimerActionListener

}
